from rivr.http import Response
from rivr.views.base import TemplateView
from rivr.middleware import Middleware

from jinja2 import Environment

class JinjaResponse(Response):
    def __init__(self, request, template_name, context, env=None, *args, **kwargs):
        super(JinjaResponse, self).__init__()
        self.request = request
        self.template_name = template_name
        self.context = context
        self._content = None
        self.jinja_env = env

        if 'request' not in self.context:
            self.context['request'] = request

    def get_env(self):
        if self.jinja_env:
            return self.jinja_env

        if hasattr(self.request, 'jinja_env'):
            return self.request.jinja_env

        raise Exception("JinjaResponse is improperly configured"
                        "and requires a jinja environment")

    def render(self):
        t = self.get_env().get_template(self.template_name)
        return t.render(self.context)

    def get_content(self):
        if not self._content:
            self._content = self.render()

        return self._content

    def set_content(self, value):
        pass
    content = property(get_content, set_content)


class JinjaView(TemplateView):
    response_class = JinjaResponse


class JinjaMiddleware(Middleware):
    def __init__(self, env, *args, **kwargs):
        self.jinja_env = env
        super(JinjaMiddleware, self).__init__(*args, **kwargs)

    def process_response(self, request, response):
        if isinstance(response, JinjaResponse):
            response.jinja_env = self.jinja_env
        return response



from rivr.sessions import BaseSessionStore

class MongoSessionStore(BaseSessionStore):
    object_id = None
    collection = 'sessions'

    def get_collection(self, request):
        return request.mongodb_database[self.collection]

    def get_session(self, request):
        session = self.get_collection(request).find_one({'key': self.session_key})
        if session:
            self.object_id = session['_id']
            self.data = session['data']

    def save(self, request):
        collection = self.get_collection(request)

        if not self.data and self.object_id: # delete our session
            self.object_id = collection.remove(self.object_id)
        else: # update our session
            if not self.session_key:
                self.generate_key()

            session = {
                "key": self.session_key,
                "data": self.data
            }

            if self.object_id:
                session['_id'] = self.object_id

            self.object_id = collection.save(session)




from rivr import template

register = template.Library()

BLOCK_CONTEXT_KEY = 'block_context'

class BlockContext(object):
    def __init__(self):
        self.blocks = {}

    def has_block(self, name):
        return name in self.blocks and len(self.blocks[name])

    def add_blocks(self, blocks):
        for name, block in blocks.iteritems():
            if name in self.blocks:
                self.blocks[name].insert(0, block)
            else:
                self.blocks[name] = [block]

    def pop(self, name):
        try:
            return self.blocks[name].pop()
        except (IndexError, KeyError):
            return None

    def push(self, name, block):
        self.blocks[name].append(block)

class ExtendsNode(template.Node):
    def __init__(self, template_name, nodelist):
        self.template_name = template_name
        self.blocks = dict([(n.name, n) for n in nodelist.get_nodes_by_type(BlockNode)])

    def get_parent(self, context):
        if 'loader' not in context:
            raise template.TemplateSyntaxError("Template loader not in context.")
        
        t = context['loader'].load_template(self.template_name)
        
        if isinstance(t, list):
            raise template.TemplateSyntaxError("%s template not found in: %s" % (self.template_name, ', '.join(t)))
        
        return template.Template(t)

    def render(self, context):
        context[BLOCK_CONTEXT_KEY] = BlockContext()
        context[BLOCK_CONTEXT_KEY].add_blocks(self.blocks)
        return self.get_parent(context).render(context)

def do_extends(parser, token):
    bits = token.contents.split('"')

    if len(bits) != 3:
        raise template.TemplateSyntaxError("Extends tag takes one argument: the template file to be included")

    nodelist = parser.parse()

    if nodelist.get_nodes_by_type(ExtendsNode):
        raise TemplateSyntaxError("'%s' cannot appear more than once in the same template" % bits[0])

    return ExtendsNode(bits[1], nodelist)
register.tag('extends', do_extends)

class BlockNode(template.Node):
    def __init__(self, name, nodelist):
        self.name = name
        self.nodelist = nodelist

    def render(self, context):
        if BLOCK_CONTEXT_KEY in context and context[BLOCK_CONTEXT_KEY].has_block(self.name):
            return context[BLOCK_CONTEXT_KEY].pop(self.name).render(context)
        return self.nodelist.render(context)

def do_block(parser, token):
    bits = token.contents.split()
    if len(bits) != 2:
        raise TemplateSyntaxError("'%s' tag takes only one argument" % bits[0])
    block_name = bits[1]

    block_nodelist = parser.parse(('endblock',))
    parser.delete_first_token()

    return BlockNode(block_name, block_nodelist)
register.tag('block', do_block)

class IncludeNode(template.Node):
    def __init__(self, template_name):
        self.template_name = template_name
    
    def render(self, context):
        if 'loader' not in context:
            raise template.TemplateSyntaxError("Template loader not in context.")
        
        t = context['loader'].load_template(self.template_name)
        
        if isinstance(t, list):
            raise template.TemplateSyntaxError("%s template not found in: %s" % (self.template_name, ', '.join(t)))
        
        t = template.Template(t)
        return t.render(context)

def do_include(parser, token):
    bits = token.contents.split('"')
    
    if len(bits) != 3:
        raise template.TemplateSyntaxError("Include tag takes one argument: the template file to be included")
    
    return IncludeNode(bits[1])
register.tag('include', do_include)



import os
from urlparse import urlparse

from rivr.middleware import Middleware

import pymongo

class MongoMiddleware(Middleware):
    database = None
    uri = None

    def get_database(self):
        if not self.database:
            if not self.uri and 'MONGODB_URI' in os.environ:
                self.uri = os.environ['MONGODB_URI']

            if not self.uri:
                raise Exception("MongoMiddleware improperly configured")

            m = urlparse(self.uri)
            self.connection = pymongo.Connection(m.hostname, m.port)

            self.database = self.connection[m.path.strip('/')]
            
            if m.username and m.password:
                self.database.authenticate(m.username, m.password)

        return self.database

    def process_request(self, request):
        request.mongodb_database = self.get_database()




class Middleware(object):
    def __init__(self, handler=None, **kwargs):
        self.handler = handler

        for key, value in kwargs.iteritems():
            setattr(self, key, value)

    def __call__(self, request):
        if hasattr(self, 'process_request'):
            response = self.process_request(request)

            if response:
                return response

        try:
            response = self.handler(request)
        except Exception, e:
            if hasattr(self, 'process_exception'):
                response = self.process_exception(request, e)

                if response:
                    return response
            else:
                raise e

        if hasattr(self, 'process_response'):
            response = self.process_response(request, response)

        return response

